function [y,z]=latc_var(x,k1,k2,zi)
% LATC_VAR  time-variant lattice all-pole filter
%   [y,z]=latc_var(x,k1,k2,zi)
%   INPUT
%     x  ... input sequence (vector)
%     k1 ... reflection coefficients (vector) for time=1
%     k2 ... reflection coefficients (vector) for time=1+length(x)
%            (length(k2) = length(k1) !!)
%            (the filter interpolates linearly between k1 and k2)
%     zi ... state vector of the filter for time=0
%            (length(zi) = length(k1) !!)
%            (default zi=zeros(1,length(k1)))
%   OUTPUT
%     y ... output sequence
%     z ... state vector of the filter for time=length(x)
% (c) 1999 by Christian Feldbauer
% feldbauer@iem.kug.ac.at



N=length(x);
p=length(k1);
if length(k2) ~= p
   error('length(k1) == length(k2) !!')
end
if nargin < 4
   zi=zeros(1,length(k1));
elseif length(zi) ~= p
   error('length(zi) == length(ki) !!')
end
k=zeros(p,N);
for i=1:p
   k(i,:)=-interp1([1,N+1],[k1(i),k2(i)],1:N);
end
z=zi;
y=zeros(1,N);
t=zeros(1,p);
for n=1:N
   t(p)=x(n)+k(p,n)*z(p);
   for j=(p-1):-1:1
      t(j)=t(j+1)+k(j,n)*z(j);
   end
   y(n)=t(1);
   for j=p:-1:2
      z(j)=z(j-1)-k(j-1,n)*t(j-1);
   end
   z(1)=t(1);
end
