% lpc_pulse_dur.m
% LPC -- pulse excitataion only
% can perform a time-scale modification
% 2002 by Christian Feldbauer
% feldbauer@tugraz.at


N = 256; % window length (even)
Ha = N; % frame hop size of analysis

TSM = 0.66; % time-scale modification factor

Hs = round( TSM*Ha); % frame hop size of synthesis

O = 16;  % order of polynom


[wav, fs, fp, fn] = get_wav;
% wav ... column vector ... samples of one channel of soundfile
L = length( wav);

w = hanning( N); % analysis window


F = floor( (L-N)/Ha ) + 1; % number of frames
par_lpc = zeros( F, O+1); % matrix for the lpc coefficients
for k = 0:F-1

    x = wav( 1 + k*Ha : N + k*Ha); % analysis window
    x = x .* w;
    
    [a, g] = lpc( x, O); % lpc analysis
    rc = tf2latc( 1, a); % convert to reflection coefficients
    
    % store coefficients for later resynthesis
    par_lpc( k+1, 1:O) = rc.';
    par_lpc( k+1, O+1) = g;
end



% resynthesis...

% generate artificial excitation:
%%%%%%%% FUNDAMENTAL FREQUENCY %%%%%%%%
%f = 100; % fundamental frequency in Hz
%pulses = zeros( (F-1)*Hs, 1);
%t = round( fs / f);
%pulses( 1:t:end) = 1; % pulsetrain with same duration like entire input signal
% note: fundamental frequency is rounded to fs/round(fs/f) for implementation simplicity


% or another excitation...
%%%%%% START AND STOP FREQUENCY %%%%%%
f_start = 120;
f_stop  = 100;
ch = chirp( 0:1/fs:((F-1)*Hs-1)/fs, f_start, ((F-1)*Hs-1)/fs, f_stop);
% peak-picking to obtain pulstrain
pick = diff( sign( diff( ch.'))) < 0;
pick = [ 0; pick; 0];
pulses = ch.' .* pick;




z = zeros( (F-1)*Hs, 1); % preallocate memory
ini = zeros( 1, O); % internal state of filter

for k = 0:F-2  % frame-by-frame
  exc = pulses( 1+k*Hs : (k+1)*Hs); % segment of pulsetrain for excitation

  % gain of two consecutive frames
  g1 = par_lpc( k+1, O+1);
  g2 = par_lpc( k+2, O+1);
  
  % linear interpolation between g1 and g2
  g = interp1( [1, Hs+1], [g1, g2], 1:Hs);
    
  exc = exc .* sqrt(g.'); % apply gain
  
  % lattice filter reflection coefficients:
  % it will be interpolated linearly  between k1 and k2 in latc_var()
  k1 = par_lpc( k+1, 1:O);
  k2 = par_lpc( k+2, 1:O);
  
  [y, ini] = latc_var( exc, k1, k2, ini); % time-varying filtering
  
  z( 1+k*Hs : (k+1)*Hs) =  y; % concatenate segments
end

z = z * 0.99 / max( abs(z)); % normalize output signal

% play output using sound(z,fs)
